@@ -21,10 +21,10 @@ from typing import TypeVar
 from typing import Callable
 from typing import Optional
 from typing import NamedTuple
+from typing import ContextManager
 from cftool.misc import safe_execute
 from cftool.misc import print_warning
 from cftool.misc import shallow_copy_dict
-from cftool.misc import context_error_handler
 from cftool.array import arr_type
 from cftool.array import save_images
 from cftool.types import tensor_dict_type
@@ -107,6 +107,29 @@ T = TypeVar("T", bound="DiffusionAPI")
 TAnnotator = TypeVar("TAnnotator", bound="Annotator")
 
 
+class SDVersions(str, Enum):
+    v1_5_BC = ""
+    v1_5 = "v1.5"
+    ANIME = "anime"
+    ANIME_ANYTHING = "anime_anything"
+    ANIME_HYBRID = "anime_hybrid"
+    DREAMLIKE = "dreamlike_v1"
+
+
+def get_sd_tag(version: SDVersions) -> str:
+    if version == SDVersions.v1_5_BC:
+        tag = "v1.5"
+    elif version == SDVersions.ANIME:
+        tag = "anime_nai"
+    elif version == SDVersions.ANIME_ANYTHING:
+        tag = "anime_anything_v3"
+    elif version == SDVersions.ANIME_HYBRID:
+        tag = "anime_hybrid_v1"
+    else:
+        tag = version
+    return f"ldm_sd_{tag}"
+
+
 class DiffusionAPI(APIMixin):
     m: DDPM
     sampler: ISampler
@@ -114,6 +137,8 @@ class DiffusionAPI(APIMixin):
     first_stage: Optional[IAutoEncoder]
     latest_seed: int
     latest_variation_seed: Optional[int]
+    sd_weights: Dict[str, tensor_dict_type]
+    current_sd_version: Optional[SDVersions]
 
     def __init__(
         self,
@@ -128,6 +153,8 @@ class DiffusionAPI(APIMixin):
         self.sampler = m.sampler
         self.cond_type = m.condition_type
         self.clip_skip = clip_skip
+        self.sd_weights = {}
+        self.current_sd_version = None
         # extracted the condition model so we can pre-calculate the conditions
         self.cond_model = m.condition_model
         if self.cond_model is not None:
@@ -179,6 +206,23 @@ class DiffusionAPI(APIMixin):
         for k, v in self._uncond_cache.items():
             self._uncond_cache[k] = v.to(device)
 
+    def prepare_sd(self, versions: List[SDVersions]) -> None:
+        for tag in map(get_sd_tag, versions):
+            if tag not in self.sd_weights:
+                self.sd_weights[tag] = torch.load(download_model(tag))
+
+    def switch_sd(self, version: SDVersions) -> None:
+        tag = get_sd_tag(version)
+        if self.current_sd_version is not None:
+            if tag == get_sd_tag(self.current_sd_version):
+                return
+        d = self.sd_weights.get(tag)
+        if d is None:
+            raise ValueError(f"cannot find tag ({tag}) in the loaded weights")
+        with self.load_context() as wrapper:
+            wrapper.load_state_dict(d)
+        self.current_sd_version = version
+
     @property
     def size_info(self) -> SizeInfo:
         opt_size = self.m.img_size
@@ -865,8 +909,8 @@ class DiffusionAPI(APIMixin):
             **kwargs,
         )
 
-    def load_context(self) -> context_error_handler:
-        class _(context_error_handler):
+    def load_context(self) -> ContextManager:
+        class _:
             def __init__(self, api: DiffusionAPI):
                 self.api = api
                 self.m_cond = api.m.condition_model
@@ -880,7 +924,7 @@ class DiffusionAPI(APIMixin):
 
                 return Wrapper(self.api.m)
 
-            def _normal_exit(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
+            def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
                 self.api.m.condition_model = self.m_cond
 
         return _(self)
@@ -1216,7 +1260,7 @@ class ControlledDiffusionAPI(DiffusionAPI):
         use_half: bool = False,
         clip_skip: int = 0,
         hint_channels: int = 3,
-        num_pool: int = 2,
+        num_pool: int = 4,
     ):
         super().__init__(
             m,
@@ -1497,6 +1541,7 @@ def ldm_semantic(pretrained: bool = True) -> DLPipeline:
 
 
 __all__ = [
+    "SDVersions",
     "DiffusionAPI",
     "ControlNetHints",
     "ControlledDiffusionAPI",
