@@ -4,6 +4,8 @@ import torch
 import torch.nn as nn
 import torch.nn.functional as F
 
+from abc import abstractmethod
+from abc import ABCMeta
 from torch import Tensor
 from typing import Any
 from typing import Dict
@@ -22,6 +24,48 @@ from ..activations import Activation
 from ....misc.toolkit import interpolate
 
 
+class ConvHook(Module, metaclass=ABCMeta):
+    @abstractmethod
+    def callback(self, net: Tensor) -> Tensor:
+        pass
+
+
+class HijackConv1d(nn.Conv1d):
+    def __init__(self, *args: Any, hook: Optional[ConvHook] = None, **kwargs: Any):
+        super().__init__(*args, **kwargs)
+        self.hook = hook
+
+    def forward(self, net: Tensor) -> Tensor:
+        net = super().forward(net)
+        if self.hook is not None:
+            net = self.hook.callback(net)
+        return net
+
+
+class HijackConv2d(nn.Conv2d):
+    def __init__(self, *args: Any, hook: Optional[ConvHook] = None, **kwargs: Any):
+        super().__init__(*args, **kwargs)
+        self.hook = hook
+
+    def forward(self, net: Tensor) -> Tensor:
+        net = super().forward(net)
+        if self.hook is not None:
+            net = self.hook.callback(net)
+        return net
+
+
+class HijackConv3d(nn.Conv3d):
+    def __init__(self, *args: Any, hook: Optional[ConvHook] = None, **kwargs: Any):
+        super().__init__(*args, **kwargs)
+        self.hook = hook
+
+    def forward(self, net: Tensor) -> Tensor:
+        net = super().forward(net)
+        if self.hook is not None:
+            net = self.hook.callback(net)
+        return net
+
+
 class GaussianBlur3(nn.Module):
     def __init__(self, in_channels: int):
         super().__init__()
@@ -529,6 +573,16 @@ class SEBlock(Module):
         return inp * net
 
 
+def conv_nd(n: int, *args: Any, **kwargs: Any) -> Module:
+    if n == 1:
+        return HijackConv1d(*args, **kwargs)
+    elif n == 2:
+        return HijackConv2d(*args, **kwargs)
+    elif n == 3:
+        return HijackConv3d(*args, **kwargs)
+    raise ValueError(f"unsupported dimensions: {n}")
+
+
 def get_conv_blocks(
     in_channels: int,
     out_channels: int,
@@ -575,6 +629,9 @@ def get_conv_blocks(
 
 
 __all__ = [
+    "HijackConv1d",
+    "HijackConv2d",
+    "HijackConv3d",
     "GaussianBlur3",
     "Conv2d",
     "DepthWiseConv2d",
@@ -585,5 +642,6 @@ __all__ = [
     "CABlock",
     "ECABlock",
     "SEBlock",
+    "conv_nd",
     "get_conv_blocks",
 ]
